# This file is placed in the Public Domain.


"wijs, wijsheid, wijs!"


import logging


from random import SystemRandom


from tob.brokers import Fleet
from tob.handler import Event
from tob.threads import Repeater


rand = SystemRandom()


def init():
    event = Event()
    repeater = Repeater(3600.0,  wsd, event)
    repeater.start()
    logging.warning("%s wise", len(TXT.split("\n")))


def wsd(event):
    Fleet.announce(rand.choice(TXT.split("\n")).strip()[2:])


TXT = """| wijsheid, wijs !

| OVERDRACHT
| ==========

| No Voice, No Description:

| enkelt eenlingen
| knipperogen
| ARC
| boot dromerij (mr_boeking)
| kwellerei
| schuldwijzerij
| klappenknijperij
| heelalknijperij
| de afleggingen
| dag
| oordeel
| zak door de grond (wederopwekking test, test = roep je er om)
| passage, voorzetterij, vang maar op
| projectie
| nieuw making (het vervolg leggende)
| hemel zakking
| ok computer
| nieuwe hemel
| nieuw buiten heersende
| dag sluiting
| uurtjes
| nazi met hel zoekerij
| peace departement
| muur stempelen
| muur duwerij
| eeuwige oordeel analyse (het toon jezelf)
| nieuwe zachtaardige
| goed oefening (inspirate geving)
| herkenning van oud goed
| woorden, woorden
| WOORD
| droom analyse
| ballen, werelden
| zone drukking
| enkelingen over
| in boek
| steek en eerste lichting evaluatie
| domeintjes
| les brenging (hemel poorten, hell legging)
| dubbelzinning praat
| mensen in den hoge
| ter verantwoording roeping (het goede woord)
| hulp roeping (melding)
| woord herkenning door licht (het begrip)
| tripping (de auto)
| mental sex (enorm lang lopen geweest, met 1e schuld legging erbij)
| tweede schuldlegging is goed gedaan op het atomaire vlak (gebied)
| derde schuldlegging is materiaal tot enkelingen gereduceerd
| vierde schuldlegging is de terug schuldlegging (schepping mishandelt)
| onschuld
| oordeel is hier het gesproken woord
| woord gebruik
| stil makerij
| constanten
| vroeger onzichtbare
| De Mens
| De Hemel
| pad terug gegeven (en nooit het gegeven pad nemen)
| iets nemen is dan een kiezen (tandarts)
| crews (besturing)
| levenden in materiaal
| levenden in hell (uitgebeeld)
| OVERDRACHT
| company, mens beheer op aarde, ziet atlen komen en gaan
| poging tot nieuw start
| de arc types (hier selecteren om einde aan de rest te noemen)
| boven gelegd woord (zeewier in vissenbak)
| het 4 leg zysteem (woord gelegd op schepping .. voor making)
| de immer zijnde dreiging tot einde
| de weigerende mens hier toe (de weerwil)
| scheppers vragen
| instappend beeld
| judgement has passed
| ring (wit/rood) als mr. evidence calling
| na de komma als uitleg, buiten lopend reciterende
| atlje coden
| eigendom discussies
| kooi houderij
| in mental verhaal houderij
| laat die 2 maar alleen (de boodschappers)
| the film is the prison
| hell als gevangenis
| oordeel millenia sluiting
| vrijlaten van alle gevangenen akkoord
| dit alles tot einde oordeel millenium
| nog meer om meneer echt niet op de knop te laten drukken
| de vragende mens, die vraagt om nog meer vragen
| trotse uitbeelding, verwoording
| constante correctheids checking
| overdracht met zonde melding
| alice trap naar de hemel uitgebeeld
| buiten koppeling getoond
| de mens zijn gemanipuleerde fantasie gebruikt als trigger van buiten fenomeen
| ook weer als fenomenen getoond
| controle over fenomenen getoond
| levenden in droom vragen om aandacht en melding
| dagjes wel of niet bij mensen in de droom
| boven hemelen vermelding
| kapot schopperij en hoe lang dat geduurt heeft
| melding voor terug gedaan om melding te ontdoen van zijn kracht
| vergeten, vergeten
| suffering on the floor
| schuld ontkenning als de tergkracht
| stekker verhaal, afscheid gabriel
| droom geheugen en reeel geheugen terug voeding
| woordleggers
| stroom tagging
| attributen beelding (twee op de schouders)
| boeken referentie, opgelegd woord als proeve
| moraal roeping
| haakse als effectief op gedachtenstroom (uitleg van de hemel)
| de een zijn hemel is de ander zijn hel
| 'het woord begeleid', nieuw komenden
| ultra addaptables
| terminator badjes
| denk instraal met materiaal (zonder handjes)
| huidig bepaling
| engine (gruis)
| essentiele vergeetachtigheid
| mental working (mogelijkheden schepping)
| attributen bepaling
| droom insluiting
| diamanten aan de onderkant van de boot
| zware oogst in de hemelen
| dag van vandaag coden (random op gegeven topics)
| oude pijn ignored
| nog niet gespoelt (nog niet verkeerd afgerond)
| de dingen liggen nog open
| stilstaande stroom (water)
| plasma
| groen-zwart acid burn
| 4 kwadranten bijna opening
| testen op de geest
| oog voorbijgaand (passerend)
| genezing belofte (het herstelt zich vanzelf)
| muziek bezoektes
| schoonmakerij
| de persoonoogst
| caretaker discussions (double their wages)
| de verkeerd noeming
| algemeenheids denken getoond (laagje erachter)
| legacy vernoemd
| de heer van de beheersing heerst hier, kuisheid
| schuld projectie is hoeks erop, uitslaan en uit besef doorslaan
| dreig projectie, torque .. de armdraai
| haaks erop de heen en weer dans, actie/reactie met verkeerd en goed vermeerderen
| false profeet omklap (linker pop die van meekijken naar inkijken woord)
| de dag na dag deze dag (vandaag)
| meervoudige oordeels voering
| materiaal reactie eromheen
| putten dempers
| zelfvertoning, duidbaar gemaakt worden
| de belofte man is er weer
| slecht advies van buiten melding
| constante redders nood melding
| de overname mensen zeggen er geen last van te hebben
| topic naming als presence
| engels/nederlands als vroegtijde dag voorkomers
| strictly dutch as The Day Preventers
| corner bouncing
| long distance woord over de ruit
| stiltewens genoemd
| de doorpraten noodzaak
| stok sturende met niet gemaaktheid als gezien (opgemerkt)
| tagging/merking is de moeilijke zaak
| schuldwijzing is de gaande zaak
| terugwijzer als immer aanwezige
| niet op mensen, niet op vogels
| brievenbus en graf
| twee eieren (gemeente huis en buiging naar west)
| vreedevernoeming
| duiding
| coding
"""
